﻿// code is Copyright (c) 2008 Julian M Bucknall, www.boyet.com
using System;

namespace RedBlackTrees {
  public class BinarySearchTree<K,V> where K : IComparable<K> {

    private BsTreeNode<K,V> root;
    public BsTreeNode<K,V> Root {
      get { return root; }
      set { root = value; }
    }

    public bool Contains(K key) {
      return GetNode(key) != null;
    }

    private BsTreeNode<K, V> GetNode(K key) {
      BsTreeNode<K, V> walker = root;
      while (walker != null) {
        BsTreeNode<K, V> nextNode = walker.Advance(key) as BsTreeNode<K, V>;
        if (nextNode == walker) {
          return nextNode;
        }
        walker = nextNode;
      }
      return null;
    }

    public void Insert(K key, V value) {
      BsTreeNode<K, V> parent = null;

      BsTreeNode<K, V> walker = root;
      while (walker != null) {
        BsTreeNode<K, V> nextNode = walker.Advance(key) as BsTreeNode<K, V>;
        if (nextNode == walker)
          throw new Exception("Binary Search Tree: inserting duplicate key");
        parent = walker;
        walker = nextNode;
      }

      walker = new BsTreeNode<K, V>(key, value);
      if (parent == null)
        root = walker;
      else if (key.CompareTo(parent.Key) < 0)
        parent.Left = walker;
      else
        parent.Right = walker;
    }


    public bool TryGetValue(K key, out V value) {
      BsTreeNode<K, V> node = GetNode(key);
      if (node == null) {
        value = default(V);
        return false;
      }
      value = node.Value;
      return true;
    }

    public V this[K key] {
      get {
        BsTreeNode<K,V> node = GetNode(key);
        if (node == null)
          throw new Exception("Binary Search Tree: Key not found using indexer");
        return node.Value;
      }
      set
      {
        BsTreeNode<K,V> node = GetNode(key);
        if (node == null)
          Insert(key, value);
        else
          node.Value = value;
      }
    }
  }
}
