﻿// code is Copyright (c) 2008 Julian M Bucknall, www.boyet.com
using System;
using System.Drawing;
using System.Windows.Forms;


namespace RedBlackTrees {
  public partial class TreePainterApp : Form {
    private BinarySearchTree<char, char> tree;
    private Bitmap bitmap;
    private char[] keyOrder = { 'p', 'g', 'w', 's', 'c', 'u', 'k', 'y', 'a', 'q', 'e', 'i', 'x', 'm', 'z' };
    private int keyPointer = 0;

    public TreePainterApp() {
      InitializeComponent();
    }

    private void btnSave_Click(object sender, EventArgs e) {
      using (SaveFileDialog saveDialog = new SaveFileDialog())
      {
        saveDialog.DefaultExt = ".jpg";
        saveDialog.OverwritePrompt = true;
        saveDialog.Title = "Save tree image";
        saveDialog.Filter = "JPG files (*.jpg)|*.jpg|All files (*.*)|*.*";
        if (saveDialog.ShowDialog() == DialogResult.OK) {
          bitmap.Save(saveDialog.FileName, System.Drawing.Imaging.ImageFormat.Jpeg);
        }
      }
    }

    private void PaintPanel(Graphics g) {
      float unit = 30.0f;
      g.ScaleTransform(unit, unit);
      GridLines.Draw(g, pnlCanvas.Width, pnlCanvas.Height, unit);

      g.TranslateTransform(1.0f, 1.0f);
      using (BsTreeNodePainter<char, char> nodePainter = new BsTreeNodePainter<char, char>(g, unit)) {
        TreePainter painter = new TreePainter(nodePainter);
        painter.DrawTree(tree.Root);
      }
    }

    private void TreePainterApp_Load(object sender, EventArgs e)
    {
      tree = new BinarySearchTree<char,char>();
    }

    private void btnAddNode_Click(object sender, EventArgs e) {
      if (keyPointer < keyOrder.Length) {
        tree.Insert(keyOrder[keyPointer], keyOrder[keyPointer]);
        keyPointer++;
        pnlCanvas.Invalidate();
      }
      else
        btnAddNode.Enabled = false;
    }

    private void pnlCanvas_Paint(object sender, PaintEventArgs e) {
      bitmap = new Bitmap(pnlCanvas.Width, pnlCanvas.Height);
      using (Graphics g = Graphics.FromImage(bitmap)) {
        g.Clear(Color.White);
        g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
        PaintPanel(g);
      }
      e.Graphics.DrawImage(bitmap, 0, 0);
    }
  }
}