// code is Copyright (c) 2008 Julian M Bucknall, www.boyet.com
using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace RedBlackTrees {
  public class SimpleNodePainter : IBinaryNodePainter, IDisposable {
    protected bool disposed = false;
    protected Graphics surface;

    protected Pen edgePen;
    protected Pen borderPen;
    protected Font font;
    protected Brush fontBrush;

    public SimpleNodePainter(Graphics g, float unit) {
      surface = g;
      edgePen = new Pen(Color.Black, 2.0f / unit);
      borderPen = new Pen(Color.Black, 2.0f / unit);
      font = new Font("Consolas", 0.5f);
      fontBrush = new SolidBrush(Color.Black);
    }

    public void Dispose() {
      Dispose(true);
      GC.SuppressFinalize(this);
    }
    protected virtual void Dispose(bool disposing) {
      if (!disposed) {
        if (disposing) {
          if (edgePen != null) edgePen.Dispose();
          if (borderPen != null) borderPen.Dispose();
          if (font != null) font.Dispose();
          if (fontBrush != null) fontBrush.Dispose();
        }
      }
      disposed = true;
    }

    public void DrawEdge(int childNumber, int childLevel,
                          int parentNumber) {
      int parentY = childLevel - 1;
      surface.DrawLine(edgePen,
                       new PointF(childNumber, childLevel),
                       new PointF(parentNumber, parentY)
                       );
    }
    protected virtual void DrawData(IBinaryNode node, int number, RectangleF nodeRect) {
      nodeRect.Offset(0.2f, 0.15f);
      surface.DrawString(number.ToString(), font, fontBrush, nodeRect);
    }
    public void DrawNode(IBinaryNode node, int number, int level) {
      RectangleF nodeRect = new RectangleF(number - 0.5f, level - 0.5f, 1.0f, 1.0f);

      using (Brush nodeFill = new LinearGradientBrush(nodeRect, Color.AliceBlue, Color.SkyBlue, LinearGradientMode.ForwardDiagonal)) {
        surface.FillEllipse(nodeFill, nodeRect);
      }
      surface.DrawEllipse(borderPen, nodeRect);
      DrawData(node, number, nodeRect);
    }
    public void DrawEdgedNode(IBinaryNode node,
                              int childNumber, int childLevel,
                              int parentNumber) {
      DrawEdge(childNumber, childLevel, parentNumber);
      DrawNode(node, childNumber, childLevel);
    }
  }
}
