// code is Copyright (c) 2008 Julian M Bucknall, www.boyet.com
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace RedBlackTrees {
  public class TreePainter {
    private int lastNodeNumber;
    private IBinaryNodePainter nodePainter;

    public TreePainter(IBinaryNodePainter nodePainter) {
      this.nodePainter = nodePainter;
      Reset();
    }

    private void Reset() {
      lastNodeNumber = -1;
    }
    private void DrawLeftSubtree(IBinaryNode node, int level) {
      if (node != null) {
        int thisNodeNumber = DrawChildren(node, level + 1);
        nodePainter.DrawEdgedNode(node, thisNodeNumber, level, lastNodeNumber + 1);
      }
    }
    private void DrawRightSubtree(IBinaryNode node, int level) {
      if (node != null) {
        int parentNodeNumber = lastNodeNumber;
        int thisNodeNumber = DrawChildren(node, level + 1);
        nodePainter.DrawEdgedNode(node, thisNodeNumber, level, parentNodeNumber);
      }
    }
    protected int DrawChildren(IBinaryNode parent, int childLevel) {
      DrawLeftSubtree(parent.Left, childLevel);
      int parentNodeNumber = ++lastNodeNumber;
      DrawRightSubtree(parent.Right, childLevel);
      return parentNodeNumber;
    }
    private void DrawRoot(IBinaryNode root) {
      if (root != null) {
        int thisNodeNumber = DrawChildren(root, 1);
        nodePainter.DrawNode(root, thisNodeNumber, 0);
      }
    }
    public void DrawTree(IBinaryNode root) {
      Reset();
      DrawRoot(root);
    }
  }
}
