﻿// code is Copyright (c) 2008 Julian M Bucknall, www.boyet.com
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RedBlackTrees;

namespace TestBinarySearchTree {

  public class Tester {
    internal int previousKey;

    public bool VerifyOrder(BsTreeNode<int, int> node) {
      if (node == null)
        return true;
      if (!VerifyOrder(node.Left as BsTreeNode<int, int>))
        return false;
      if (node.Key.CompareTo(previousKey) <= 0)
        return false;
      return VerifyOrder(node.Right as BsTreeNode<int, int>);
    }

    public void Run() {
      Console.WriteLine("Testing binary search tree implementation");

      Console.WriteLine("Creating BSTree");
      BinarySearchTree<int, int> tree = new BinarySearchTree<int, int>();

      Console.WriteLine("Adding 2000 random values, and verifying tree structure...");
      Random rg = new Random();
      int key;
      for (int i = 0; i < 2000; i++) {
        do {
          key = rg.Next(300000);

        } while (tree.Contains(key));
        tree.Insert(key, key + 1);
        if (!tree.Contains(key)) {
          Console.WriteLine("FAIL: new key not found");
          Console.ReadLine();
        }
        int value = tree[key];
        if (value != key + 1) {
          Console.WriteLine("FAIL: cannot retrieve value for key");
          Console.ReadLine();
        }
        tree[key] = value + 1;
        value = tree[key];
        if (value != key + 2) {
          Console.WriteLine("FAIL: cannot retrieve new value for key");
          Console.ReadLine();
        }
        tree[key + 300000] = value + 1;
        value = tree[key + 300000];
        if (value != key + 3) {
          Console.WriteLine("FAIL: cannot retrieve new value for extra key");
          Console.ReadLine();
        }
        previousKey = -1;
        if (!VerifyOrder(tree.Root)) {
          Console.WriteLine("FAIL: keys out of order");
          Console.ReadLine();
        }
      }
      Console.WriteLine("...done.");
    }
  }


  public class Program {

    static void Main(string[] args) {
      Tester tester = new Tester();
      tester.Run();

      Console.ReadLine();
    }
  }
}
